WQAchievements = LibStub("AceAddon-3.0"):NewAddon("WQAchievements", "AceConsole-3.0", "AceTimer-3.0")
WQA = WQAchievements

-- Blizzard
local IsActive = C_TaskQuest.IsActive
local GetQuestInfoByQuestID = C_TaskQuest.GetQuestInfoByQuestID

-- Defaults
local defaults = {
	char = {
		achievements = {
			pvp = {id = 11474, status = true},
			ff = {id = 9696, status = true},
			botbi = {id = 10876, status = true},
			fishing = {id = 10598, status = true}
		},
		toast = true,
		chat = true
	}
}

-- Locales
local locale = GetLocale()
WQA.L = {}
local L = WQA.L
L["WQToast"] = "Interesting World Quest"
L["WQChat"] = "Interesting World Quests are active:"
if locale == "deDE" then
	L["WQToast"] = "Interessante Weltquest"
	L["WQChat"] = "Interessante Weltquests sind verfügbar:"
end

WQA:RegisterChatCommand("wqa", "slash")

function WQA:slash(input)
	local arg1 = string.lower(input)

	if arg1 == "" then
		self:checkWQ()
	elseif arg1 == "details" then
		self:checkWQ("details")
	elseif self.db.char.achievements[arg1] then
		print("Tracking of "..GetAchievementLink(self.db.char.achievements[arg1].id).." is now "..tostring(not self.db.char.achievements[arg1].status))
		self.db.char.achievements[arg1].status = not self.db.char.achievements[arg1].status
		self:buildQList()
	elseif arg1 == "chat" then
		self.db.char.chat = not self.db.char.chat
	elseif arg1 == "?" then
		for k,v in pairs(self.db.char.achievements) do
			print("Tracking of "..GetAchievementLink(v.id).." is "..tostring(v.status))
			print("Toggle tracking with /wqa "..k)
		end
		print("Toggle chat output with /wqa chat")
	end
end

WQA.aL = {}
WQA.aL.pvp = {11475, 11476, 11477, 11478}
WQA.aL.ff = {9686, 9687, 9688, 9689, 9690, 9691, 9692, 9693, 9694, 9695}
WQA.aL.botbi = {10876}
WQA.aL.fishing = {10598}
WQA.trainerList = {42159, 40299, 40277, 42442, 40298, 40280, 40282, 41687, 40278, 41944, 41895, 40337, 41990, 40279, 41860}
WQA.fishQList = {}
WQA.fishQList[1] = {41612, 41613, 41270}
WQA.fishQList[3] = {41604, 41605, 41279}
WQA.fishQList[4] = {41598, 41599, 41264}
WQA.fishQList[7] = {41611, 41265, 41610}
WQA.fishQList[8] = {41617, 41280, 41616}
WQA.fishQList[9] = {41597, 41244, 41596}
WQA.fishQList[10] = {41602, 41274, 41603}
WQA.fishQList[11] = {41609, 41243}
WQA.fishQList[14] = {41615, 41275, 41614}
WQA.fishQList[19] = {41269, 41600, 41601}
WQA.watched = {}

function WQA:buildQList()
	self.questList = {}
	for k,v in pairs(self.db.char.achievements) do
		if v.status == true then
			for _,id in pairs(self.aL[k]) do
				local _,_,_,completed = GetAchievementInfo(id)
				if completed == false then
					for i=1, GetAchievementNumCriteria(id) do
						local _,t,completed,_,_,_,_,questID = GetAchievementCriteriaInfo(id,i)
						if completed == false then
							if id == 10598 and t == 0 then
								for _,questID in pairs(self.fishQList[i]) do
							 		if not self.questList[questID] then self.questList[questID] = {} end
							 		local l = self.questList[questID]
									l[#l + 1] = id
								end
							else
								if t == 158 then questID = self.trainerList[i] end
								
								if not self.questList[questID] then self.questList[questID] = {} end
								local l = self.questList[questID]
								l[#l + 1] = id
							end
						end
					end
				end
			end
		end
	end
end

function WQA:checkWQ(mode)
	local first = false
	for questID,qList in pairs(self.questList) do
		if IsActive(questID) and not (self.watched[questID] and mode == "new") then
			if IsAddOnLoaded("ls_Toasts") and self.db.char.toast then
				self:Toast(questID, qList[1])
				self.watched[questID] = true
			end
			if self.db.char.chat or not self.db.char.toast then
				if first == false then
					first = true
					print(L["WQChat"])
				end
				self.watched[questID] = true
				print(GetQuestLink(questID))
			end
			if mode == "details" then
				if first == false then
					first = true
					print(L["WQChat"])
				end
				print(GetQuestLink(questID))
				for _,aID in pairs(qList) do
					print("     "..GetAchievementLink(aID))
				end
			end
		end
	end
end

function WQA:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("WQADB", defaults)
end

-----------------
-- ls: Toasts  --
-----------------
function WQA:RegisterToast()
	local E = _G["ls_Toasts"][1]
	local C = _G["ls_Toasts"][2]
	local lsL = _G["ls_Toasts"][3]

	local function Toast_OnClick(self)
		if self._data then
			if not AchievementFrame then
				AchievementFrame_LoadUI()
			end

			if AchievementFrame then
				ShowUIPanel(AchievementFrame)
				AchievementFrame_SelectAchievement(self._data.ach_id)
			end
		end
	end

	local function Toast_SetUp(event, questID, achievementID)
		local toast = E:GetToast()
		local name = GetQuestInfoByQuestID(questID)
		local _,_,_,_,_,_,_,_,_, icon = GetAchievementInfo(achievementID)

		toast.Title:SetText(L["WQToast"])
		toast.Text:SetText(name)
		toast.Icon:SetTexture(icon)
		toast.IconBorder:Show()

		toast._data = {
			event = event,
			ach_id = achievementID,
		}

		toast:HookScript("OnClick", Toast_OnClick)
		toast:Spawn(C.db.profile.types.wqa.dnd)
	end

	function self:Toast(questID, achievementID)
		Toast_SetUp("WQA", questID, achievementID)
	end

	local function Enable()
		if C.db.profile.types.wqa.enabled then
			self.db.char.toast = true
		end
	end

	local function Disable()
		self.db.char.toast = false
	end

	local function Test()
		WQA:Toast(41269, 10598)
		WQA:Toast(42159, 10876)
		WQA:Toast(41896	, 11475)
	end

	E:RegisterOptions("wqa", {
		enabled = self.db.char.toast,
		dnd = false,
	}, {
		name = "WQAchievements",
		args = {
			enabled = {
				order = 1,
				type = "toggle",
				name = lsL["ENABLE"],
				get = function()
					return C.db.profile.types.wqa.enabled
				end,
				set = function(_, value)
					C.db.profile.types.wqa.enabled = value

					if value then
						Enable()
					else
						Disable()
					end
				end
			},
			dnd = {
				order = 2,
				type = "toggle",
				name = lsL["DND"],
				desc = lsL["DND_TOOLTIP"],
				get = function()
					return C.db.profile.types.wqa.dnd
				end,
				set = function(_, value)
					C.db.profile.types.wqa.dnd = value

					if value then
						Enable()
					else
						Disable()
					end
				end
			},
			test = {
				type = "execute",
				order = 99,
				width = "full",
				name = lsL["TEST"],
				func = Test,
			},
		},
	})

	E:RegisterSystem("wqa", Enable, Disable, Test)
end

function WQA:OnEnable()
	if IsAddOnLoaded("ls_Toasts") then
		self:RegisterToast()
	end

	self.event = CreateFrame("Frame")
	self.event:RegisterEvent("PLAYER_ENTERING_WORLD")
	self.event:SetScript("OnEvent", function ()
		self.event:UnregisterEvent("PLAYER_ENTERING_WORLD")
		self:ScheduleTimer("buildQList", 10)
		self:ScheduleTimer("checkWQ", 11)
	end)

	self:ScheduleRepeatingTimer("checkWQ",300,"new")
end